@@ -11,6 +11,7 @@ import pybedtools
 
 import settings
 
+BUFSIZE = 1
 
 _tags = {}
 
@@ -105,7 +106,14 @@ class Error(Exception):
 
 
 class BEDToolsError(Error):
-    pass
+    def __init__(self, cmd, msg):
+        self.cmd = str(cmd)
+        self.msg = str(msg)
+
+    def __str__(self):
+        m = '\nCommand was:\n\n\t' + self.cmd + '\n' + \
+            '\nError message was:\n' + self.msg
+        return m
 
 
 def isBGZIP(fn):
@@ -136,13 +144,16 @@ def isBAM(fn):
 
     cmds = ['samtools', 'view', '-H', fn]
     try:
-        p = subprocess.Popen(
-            cmds, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        stdout, stderr = p.communicate()
-        if stderr:
-            return False
 
+        # Silence the output, we want to check the return code
+        with open(os.devnull, "w") as out:
+            subprocess.check_call(cmds, stdout=out, stderr=out)
         return True
+
+    except subprocess.CalledProcessError:
+        # Non-0 return code, it means we have an error
+        return False
+
     except OSError:
         raise OSError(
             'SAMtools (http://samtools.sourceforge.net/) '
@@ -288,12 +299,17 @@ def cleanup(verbose=False, remove_all=False):
 
 
 def _version_2_15_plus_names(prog_name):
+    if not settings._bedtools_installed:
+        _check_for_bedtools()
+    if not settings._v_2_15_plus:
+        return [prog_name]
     try:
         prog_name = settings._prog_names[prog_name]
     except KeyError:
         if prog_name in settings._new_names:
             pass
-        raise BEDToolsError('"%s" not a recognized BEDTools program' % prog_name)
+        raise BEDToolsError(
+            prog_name, prog_name + 'not a recognized BEDTools program')
     return [os.path.join(settings._bedtools_path, 'bedtools'), prog_name]
 
 
@@ -334,7 +350,7 @@ def call_bedtools(cmds, tmpfn=None, stdin=None, check_stderr=None):
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
                                  stdin=subprocess.PIPE,
-                                 bufsize=1)
+                                 bufsize=BUFSIZE)
             for line in stdin:
                 p.stdin.write(line)
 
@@ -355,7 +371,7 @@ def call_bedtools(cmds, tmpfn=None, stdin=None, check_stderr=None):
                                  stdout=outfile,
                                  stderr=subprocess.PIPE,
                                  stdin=subprocess.PIPE,
-                                 bufsize=1)
+                                 bufsize=BUFSIZE)
             if isinstance(stdin, file):
                 stdout, stderr = p.communicate(stdin.read())
             else:
@@ -375,7 +391,7 @@ def call_bedtools(cmds, tmpfn=None, stdin=None, check_stderr=None):
             p = subprocess.Popen(cmds,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
-                                 bufsize=1)
+                                 bufsize=BUFSIZE)
             output = p.stdout
             stderr = None
 
@@ -390,7 +406,7 @@ def call_bedtools(cmds, tmpfn=None, stdin=None, check_stderr=None):
             p = subprocess.Popen(cmds,
                                  stdout=outfile,
                                  stderr=subprocess.PIPE,
-                                 bufsize=1)
+                                 bufsize=BUFSIZE)
             stdout, stderr = p.communicate()
             output = tmpfn
             outfile.close()
@@ -404,12 +420,7 @@ def call_bedtools(cmds, tmpfn=None, stdin=None, check_stderr=None):
                 stderr = None
 
         if stderr:
-            sys.stderr.write('\nCommand was:\n\n\t%s\n' %
-                             subprocess.list2cmdline(cmds))
-            sys.stderr.write('\nError message was:\n')
-            sys.stderr.write(stderr)
-            raise BEDToolsError('Error message from BEDTools written to '
-                                'stderr, above', stderr)
+            raise BEDToolsError(subprocess.list2cmdline(cmds), stderr)
 
     except (OSError, IOError) as err:
         print '%s: %s' % (type(err), os.strerror(err.errno))
@@ -488,6 +499,8 @@ def _check_sequence_stderr(x):
     """
     if x.startswith('index file'):
         return True
+    if x.startswith("WARNING"):
+        return True
     return False
 
 
@@ -509,16 +522,18 @@ def _call_randomintersect(_self, other, iterations, intersect_kwargs,
     )
 
 
-def close_or_delete(x):
+def close_or_delete(*args):
     """
     Single function that can be used to get rid of a BedTool, whether it's a
     streaming or file-based version.
     """
-    if isinstance(x.fn, basestring):
-        os.unlink(x.fn)
-    elif hasattr(x.fn, 'close'):
-        x.fn.close()
-
+    for x in args:
+        if isinstance(x.fn, basestring):
+            os.unlink(x.fn)
+        elif hasattr(x.fn, 'close'):
+            x.fn.close()
+        if hasattr(x.fn, 'throw'):
+            x.fn.throw(StopIteration)
 
 def _jaccard_output_to_dict(s, **kwargs):
     """
@@ -535,6 +550,7 @@ def _jaccard_output_to_dict(s, **kwargs):
     data[0] = int(data[0])
     data[1] = int(data[1])
     data[2] = float(data[2])
+    data[3] = int(data[3])
     return dict(zip(header, data))
 
 
@@ -568,4 +584,108 @@ def _reldist_output_handler(s, **kwargs):
     return results
 
 
+def n_open_fds():
+    pid = os.getpid()
+    procs = subprocess.check_output(
+        ['lsof', '-w', '-Ff', '-p', str(pid)])
+    nprocs = 0
+    for i in procs.splitlines():
+        if i[1:].isdigit() and i[0] == 'f':
+            nprocs += 1
+    return nprocs
+
+
+import re
+coord_re = re.compile(
+    r"""
+    (?P<chrom>.+):
+    (?P<start>\d+)-
+    (?P<stop>\d+)
+    (?:\[(?P<strand>.)\])?""", re.VERBOSE)
+
+
+def string_to_interval(s):
+    """
+    Convert string of the form "chrom:start-stop" or "chrom:start-stop[strand]"
+    to an interval.
+
+    Assumes zero-based coords.
+
+    If it's already an interval, then return it as-is.
+    """
+    if isinstance(s, basestring):
+        m = coord_re.search(s)
+        if m.group('strand'):
+            return pybedtools.create_interval_from_list([
+                m.group('chrom'),
+                m.group('start'),
+                m.group('stop'),
+                '.',
+                '0',
+                m.group('strand')])
+        else:
+            return pybedtools.create_interval_from_list([
+                m.group('chrom'),
+                m.group('start'),
+                m.group('stop'),
+            ])
+    return s
+
+
+class FisherOutput(object):
+    def __init__(self, s, **kwargs):
+        """
+        fisher returns text results like::
+
+            # Contingency Table
+            #_________________________________________
+            #           | not in -b    | in -b        |
+            # not in -a | 3137160615   | 503          |
+            #     in -a | 100          | 46           |
+            #_________________________________________
+            # p-values for fisher's exact test
+            left	right	two-tail	ratio
+            1.00000	0.00000	0.00000	2868973.922
+
+        """
+        if isinstance(s, str):
+            s = open(s).read()
+        if hasattr(s, 'next'):
+            s = ''.join(i for i in s)
+        table = {
+            'not in -a': {
+                'not in -b': None,
+                'in -b': None
+            },
+            'in -a': {
+                'not in -b': None,
+                'in -b': None,
+            },
+        }
+        self.text = s
+        lines = s.splitlines()
+        for i in lines:
+            if 'not in -a' in i:
+                _, not_in_b, in_b, _= i.strip().split('|')
+                table['not in -a']['not in -b'] = int(not_in_b)
+                table['not in -a']['in -b'] = int(in_b)
+
+            if '    in -a' in i:
+                _, not_in_b, in_b, _ = i.strip().split('|')
+                table['in -a']['not in -b'] = int(not_in_b)
+                table['in -a']['in -b'] = int(in_b)
+        self.table = table
+        left, right, two_tail, ratio = lines[-1].split()
+        self.left_tail = float(left)
+        self.right_tail = float(right)
+        self.two_tail = float(two_tail)
+        self.ratio = float(ratio)
+
+    def __str__(self):
+        return self.text
+
+    def __repr__(self):
+        return '<%s at %s>\n%s' % (self.__class__.__name__, id(self), self.text)
+
+
 atexit.register(cleanup)
