@@ -1,5 +1,6 @@
 import pybedtools
 import os, difflib, sys
+from textwrap import dedent
 from nose import with_setup
 from nose.tools import assert_raises, raises
 from pybedtools.helpers import BEDToolsError
@@ -124,7 +125,14 @@ def test_tabix():
         if os.path.exists(fn):
             os.unlink(fn)
 
+def test_tabix_intervals():
+    a = pybedtools.BedTool('chr1 25 30', from_string=True).tabix()
+    assert len(a.tabix_intervals('chr1:30-35')) == 0
+    assert len(a.tabix_intervals('chr1:29-30')) == 1
 
+    # make sure it works OK even if strand was provided
+    assert len(a.tabix_intervals('chr1:30-35[-]')) == 0
+    assert len(a.tabix_intervals('chr1:29-30[-]')) == 1
 
 # ----------------------------------------------------------------------------
 # Streaming and non-file BedTool tests
@@ -1059,12 +1067,76 @@ def test_extend_fields():
     results = str(a.each(featurefuncs.extend_fields, 8))
     print results
     assert results == fix("""
-    chr1	1	100	feature1	0	+	.	.
-    chr1	100	200	feature2	0	+	.	.
-    chr1	150	500	feature3	0	-	.	.
-    chr1	900	950	feature4	0	+	.	.
+    chr1	1	100	feature1	0	+	1	100
+    chr1	100	200	feature2	0	+	100	200
+    chr1	150	500	feature3	0	-	150	500
+    chr1	900	950	feature4	0	+	900	950
     """)
 
+def test_gff2bed():
+    a = pybedtools.example_bedtool('d.gff')
+    results = str(a.each(featurefuncs.gff2bed, name_field='Parent'))
+    assert results == fix("""
+    chr1	49	300	.	.	+
+    chr1	49	300	gene1	.	+
+    chr1	74	150	mRNA1	.	+
+    chr1	199	275	mRNA1	.	+
+    chr1	1199	1275	.	.	+""")
+
+
+    results = str(a.each(featurefuncs.gff2bed))
+    assert results == fix("""
+    chr1	49	300	gene1	.	+
+    chr1	49	300	mRNA1	.	+
+    chr1	74	150	CDS1	.	+
+    chr1	199	275	CDS2	.	+
+    chr1	1199	1275	rRNA1	.	+
+    """)
+
+    results = str(a.each(featurefuncs.gff2bed, name_field="nonexistent"))
+    assert results == fix("""
+    chr1	49	300	.	.	+
+    chr1	49	300	.	.	+
+    chr1	74	150	.	.	+
+    chr1	199	275	.	.	+
+    chr1	1199	1275	.	.	+
+    """)
+
+    results = str(a.each(featurefuncs.gff2bed, name_field=1))
+    print results
+    assert results == fix("""
+    chr1	49	300	fake	.	+
+    chr1	49	300	fake	.	+
+    chr1	74	150	fake	.	+
+    chr1	199	275	fake	.	+
+    chr1	1199	1275	fake	.	+""")
+
+
+def test_add_color():
+    try:
+        from matplotlib import cm
+    except ImportError:
+        print "matplotlib not installed; skipping test_add_color"
+        return
+
+    def modify_scores(f):
+        fields = f.fields
+        fields[4] = str(f[2])
+        return pybedtools.create_interval_from_list(fields)
+    a = pybedtools.example_bedtool('a.bed')
+    a = a.each(modify_scores).saveas()
+    cmap = cm.jet
+    norm = a.colormap_normalize()
+    results = str(a.each(featurefuncs.add_color, cmap=cmap, norm=norm))
+    print results
+    assert results == fix("""
+    chr1	1	100	feature1	100	+	1	100	0,0,127
+    chr1	100	200	feature2	200	+	100	200	0,0,255
+    chr1	150	500	feature3	500	-	150	500	99,255,147
+    chr1	900	950	feature4	950	+	900	950	127,0,0""")
+
+
+
 #------------------------------------------------------------------------------
 # Tests for IntervalFile, as accessed by BedTool objects
 #------------------------------------------------------------------------------
@@ -1165,6 +1237,10 @@ def test_window_maker():
     chr1	450	500
     chr1	900	950
     """)
+    x = pybedtools.BedTool()
+    z = x.window_maker(genome='hg19', w=100000)
+    assert str(z[0]) == "chr1\t0\t100000\n"
+    assert str(z[10000]) == 'chr16\t20800000\t20900000\n'
 
 def test_random():
     a = pybedtools.BedTool()
@@ -1210,11 +1286,6 @@ def test_bam_to_fastq():
     y = x.bam_to_fastq(fq=tmpfn)
     assert open(y.fastq).read() == open(pybedtools.example_filename('small.fastq')).read()
 
-def test_window():
-    x = pybedtools.BedTool()
-    z = x.window_maker(genome='hg19', w=100000)
-    assert str(z[0]) == "chr1\t0\t100000\n"
-    assert str(z[10000]) == 'chr16\t20800000\t20900000\n'
 
 def test_gtf_gff_attrs():
     # smoke test.
@@ -1242,7 +1313,7 @@ def test_jaccard():
     x = pybedtools.example_bedtool('a.bed')
 
     results = x.jaccard(pybedtools.example_bedtool('b.bed'))
-    assert results == {'intersection': 46, 'union': 649, 'jaccard': 0.0708783}, results
+    assert results == {'intersection': 46, 'union-intersection': 649, 'jaccard': 0.0708783, 'n_intersections': 2}, results
 
     results2 = x.jaccard(pybedtools.example_bedtool('b.bed'), stream=True)
     assert results == results2, results2
@@ -1258,3 +1329,195 @@ def test_reldist():
     chr1	1	100	feature1	0	+	0.282
     chr1	100	200	feature2	0	+	0.153
     chr1	150	500	feature3	0	-	0.220""")
+
+def test_remote_bam():
+    x = pybedtools.BedTool(
+        ('ftp://ftp-trace.ncbi.nih.gov/1000genomes/ftp/data/HG00096/'
+         'exome_alignment/HG00096.chrom11.ILLUMINA.bwa.GBR.exome.'
+         '20120522.bam'),
+        remote=True)
+    def gen():
+        for i, f in enumerate(x.bam_to_bed(stream=True)):
+            yield f
+            if i == 9:
+                break
+    results = pybedtools.BedTool(gen()).saveas()
+    assert results == fix("""
+11	60636	60736	SRR081241.13799221/1	0	+
+11	60674	60774	SRR077487.5548889/1	0	+
+11	60684	60784	SRR077487.12853301/1	0	+
+11	60789	60889	SRR077487.5548889/2	0	-
+11	60950	61050	SRR077487.13826494/1	0	+
+11	60959	61059	SRR081241.13799221/2	0	-
+11	61052	61152	SRR077487.12853301/2	0	-
+11	61548	61648	SRR081241.16743804/2	0	+
+11	61665	61765	SRR081241.16743804/1	0	-
+11	61989	62089	SRR077487.167173/2	0	+"""), results
+
+
+def test_empty_overloaded_ops():
+    a = pybedtools.example_bedtool('a.bed')
+    b = pybedtools.BedTool("", from_string=True)
+    assert b.file_type == 'empty'
+
+    # NOTE: change in semantics.  Previously, intersecting a BED file with an
+    # empty file would return the original BED file.
+    assert (a + b) == b
+    assert (b + a) == b
+    assert (a - b) == a
+    assert (b - a) == b
+    assert (b - b) == b
+
+def test_issue_81():
+    genome = {'chr1': (0, 5000)}
+    result = pybedtools.BedTool().window_maker(genome=genome, w=1000, s=500)
+    assert result == fix(
+        """
+        chr1	0	1000
+        chr1	500	1500
+        chr1	1000	2000
+        chr1	1500	2500
+        chr1	2000	3000
+        chr1	2500	3500
+        chr1	3000	4000
+        chr1	3500	4500
+        chr1	4000	5000
+        chr1	4500	5000
+        """), result
+
+def test_to_dataframe():
+    def fix_dataframe(df):
+        return ''.join(df.splitlines(True)[1:])
+    try:
+        import pandas
+    except ImportError:
+        print("pandas not installed; skipping test")
+
+    a = pybedtools.example_bedtool('a.bed')
+
+    results = str(a.to_dataframe())
+
+
+
+    expected = fix_dataframe("""
+  chrom  start  end      name  score strand
+0  chr1      1  100  feature1      0      +
+1  chr1    100  200  feature2      0      +
+2  chr1    150  500  feature3      0      -
+3  chr1    900  950  feature4      0      +""")
+    assert results == expected
+
+    d = pybedtools.example_bedtool('d.gff')
+    results = str(d.to_dataframe())
+    expected = fix_dataframe("""
+  seqname source feature  start   end score strand frame  \\
+0    chr1   fake    gene     50   300     .      +     .   
+1    chr1   fake    mRNA     50   300     .      +     .   
+2    chr1   fake     CDS     75   150     .      +     .   
+3    chr1   fake     CDS    200   275     .      +     .   
+4    chr1   fake    rRNA   1200  1275     .      +     .   
+
+               attributes  
+0                ID=gene1  
+1  ID=mRNA1;Parent=gene1;  
+2   ID=CDS1;Parent=mRNA1;  
+3   ID=CDS2;Parent=mRNA1;  
+4               ID=rRNA1;  """)
+    assert results == expected
+
+
+    # get a gff file with too many fields...
+    x = pybedtools.example_bedtool('c.gff')
+    x = x.intersect(x, c=True)
+    assert_raises(ValueError, x.to_dataframe)
+
+    names = ['seqname', 'source', 'feature', 'start', 'end', 'score', 'strand',
+             'frame', 'attributes', 'count']
+    results = str(x.to_dataframe(names=names))
+    expected = fix_dataframe("""
+   seqname source feature  start   end score strand frame  \\
+0     chr1    ucb    gene    465   805     .      +     .   
+1     chr1    ucb    gene    631   899     .      +     .   
+2     chr1    ucb    mRNA    631   913     .      +     .   
+3     chr1    ucb     CDS    760   913     .      +     .   
+4     chr1    ucb     CDS    486   605     .      +     .   
+5     chr1    ucb     CDS    706  1095     .      +     .   
+6     chr1    ucb     CDS    174   326     .      +     .   
+7     chr1    ucb     CDS    439   630     .      +     .   
+8     chr1    ucb    mRNA    496   576     .      +     .   
+9     chr1    ucb    mRNA    486   605     .      +     .   
+10    chr1    ucb    mRNA    706   895     .      +     .   
+11    chr1    ucb    mRNA    174   326     .      +     .   
+12    chr1    ucb    mRNA    439   899     .      +     .   
+13    chr1    ucb    gene     60   269     .      -     .   
+
+                                           attributes  count  
+0   ID=thaliana_1_465_805;match=scaffold_801404.1;...     11  
+1         ID=AT1G01010;Name=AT1G01010;rname=AT1G01010      7  
+2   ID=AT1G01010.mRNA;Parent=AT1G01010;rname=AT1G0...      7  
+3               Parent=AT1G01010.mRNA;rname=AT1G01010      7  
+4               Parent=AT1G01010.mRNA;rname=AT1G01010      6  
+5               Parent=AT1G01010.mRNA;rname=AT1G01010      7  
+6               Parent=AT1G01010.mRNA;rname=AT1G01010      3  
+7               Parent=AT1G01010.mRNA;rname=AT1G01010      6  
+8   ID=AT1G01010.mRNA;Parent=AT1G01010;rname=AT1G0...      6  
+9   ID=AT1G01010.mRNA;Parent=AT1G01010;rname=AT1G0...      6  
+10  ID=AT1G01010.mRNA;Parent=AT1G01010;rname=AT1G0...      7  
+11  ID=AT1G01010.mRNA;Parent=AT1G01010;rname=AT1G0...      3  
+12  ID=AT1G01010.mRNA;Parent=AT1G01010;rname=AT1G0...     11  
+13  ID=thaliana_1_6160_6269;match=fgenesh1_pg.C_sc...      3  """)
+    assert results == expected
+
+
+
+
+def test_tail():
+    a = pybedtools.example_bedtool('rmsk.hg18.chr21.small.bed')
+    observed = a.tail(as_string=True)
+    expected = fix(
+        """
+        chr21	13355834	13356047	MER58A	892	-
+        chr21	13356250	13356290	AT_rich	26	+
+        chr21	13356358	13356381	AT_rich	23	+
+        chr21	13356571	13356910	L2	333	-
+        chr21	13357179	13357987	L1MEc	1264	-
+        chr21	13358003	13358300	L1MEc	379	-
+        chr21	13358304	13358952	L1MEc	1271	-
+        chr21	13358960	13359288	L2	336	+
+        chr21	13359444	13359751	AluY	2337	+
+        chr21	13360044	13360225	L1M5	284	-""")
+    assert observed == expected
+
+
+    # only ask for 3 lines
+    observed = a.tail(3, as_string=True)
+    expected = fix(
+        """
+        chr21	13358960	13359288	L2	336	+
+        chr21	13359444	13359751	AluY	2337	+
+        chr21	13360044	13360225	L1M5	284	-""")
+    assert observed == expected
+
+
+    # For short files, whole thing should be returned
+    a = pybedtools.example_bedtool('a.bed')
+    expected = str(a)
+    obs = a.tail(as_string=True)
+    assert obs == expected
+
+def test_fisher():
+    a = pybedtools.example_bedtool('a.bed')
+    b = pybedtools.example_bedtool('b.bed')
+    c = a.fisher(b, genome='hg19')
+    assert str(c) == \
+"""# Contingency Table
+#_________________________________________
+#           | not in -b    | in -b        |
+# not in -a | 3137160615   | 503          |
+#     in -a | 100          | 46           |
+#_________________________________________
+# p-values for fisher's exact test
+left	right	two-tail	ratio
+1.00000	0.00000	0.00000	2868973.922
+""", c
+
